package gridsim.gssim;

import java.util.ArrayList;
import java.util.List;

import org.joda.time.DateTimeUtilsExt;

import gridsim.GridSim;
import gridsim.Gridlet;
import gridsim.ResGridlet;
import gssim.schedframe.scheduling.ExecTaskInterface;

/**
 * 
 * @author Marcin Krystek
 *
 * @param <T>
 */
public abstract class AbstractResGridlet<T> extends ResGridlet implements ExecTaskInterface{

	//protected List<Map<ResourceParameterName, ResourceUnit>> usedResources;
	protected List<ResourceHistoryItem> usedResources;
	
	protected AbstractResGridlet(Gridlet gridlet) {
		super(gridlet);
		this.usedResources = new ArrayList<ResourceHistoryItem>();
	}

	//public void addUsedResources(Map<ResourceParameterName, ResourceUnit> usedResources){
	public void addUsedResources(ResourceHistoryItem usedResources){
		this.usedResources.add(usedResources);
	}
	
	//public List<Map<ResourceParameterName, ResourceUnit>> getUsedResources(){
	public List<ResourceHistoryItem> getUsedResources(){
		return this.usedResources;
	}
	
	public boolean setGridletStatus(int status)
    {
        // gets Gridlet's previous status
        int prevStatus = gridlet_.getGridletStatus();

        // if the status of a Gridlet is the same as last time, then ignore
        if (prevStatus == status) {
            return false;
        }

        boolean success = true;
        try
        {
            double clock = GridSim.clock();   // gets the current clock
            long sec = DateTimeUtilsExt.currentTimeMillis() / 1000; // time in seconds 
            clock = Long.valueOf(sec).doubleValue();
            
            // sets Gridlet's current status
            gridlet_.setGridletStatus(status);

            // if a previous Gridlet status is INEXEC
            if (prevStatus == Gridlet.INEXEC)
            {
                // and current status is either CANCELED, PAUSED or SUCCESS
                if (status == Gridlet.CANCELED || status == Gridlet.PAUSED ||
                    status == Gridlet.SUCCESS)
                {
                    // then update the Gridlet completion time
                    totalCompletionTime_ += (clock - startExecTime_);
                    index_ = 0;
                    return true;
                }
            }

            if (prevStatus == Gridlet.RESUMED && status == Gridlet.SUCCESS)
            {
                // then update the Gridlet completion time
                totalCompletionTime_ += (clock - startExecTime_);
                return true;
            }

            // if a Gridlet is now in execution
            if (status == Gridlet.INEXEC ||
                (prevStatus == Gridlet.PAUSED && status == Gridlet.RESUMED) )
            {
                startExecTime_ = clock;
                gridlet_.setExecStartTime(startExecTime_);
            }
        }
        catch(Exception e) {
            success = false;
        }

        return success;
    }
	
	protected void init() {
		super.init();
		double time =  DateTimeUtilsExt.getDateTimeAt(arrivalTime_).getMillis() / 1000;
		this.gridlet_.setSubmissionTime(time);
	 }
	
}
